﻿using System;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Management;
using System.IO.Ports;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

/* Next Steps:
- Select Com Port with HMC String
- Bessere Anordnung der Symbole
- Funktionen für Spannungs- und Stromwerte

*/


namespace HMC8012Remote
{
    public partial class Form1 : Form
    {
        static Timer myTimer = new Timer();
        int chartindex, zoom_factor; 
        bool roll=false, read_HMCVer=false, DataAquisition=false;
        string answer = "", ComPort;
        decimal min, max, MinDisplayed, MaxDisplayed, Temperature, rn=1000, d, AVG_Sum, AVG;
        double initialZoomValueX, initialZoomValueY;
        //-------------------------- Measure Modes ------------------------------
        string MeasureMode, MUnit;


        //------------------------ HMC8012 Commands -----------------------------

        const string DispMsg        = "DISP:TEXT";                      // Text Message must be added like this ""Verbunden"" + "\n"
        const string DispTextClear  = "DISP:TEXT:CLE?"      + "\n";     // Clears text message in display
        
        // Meas requests a new measurement from Device, this takes apprx. 330 ms, faster is to require results with FETCh? Command, This gets value from Display 

        const string SetMeasDCV     = "CONF:VOLT:DC"       + "\n";     // Sets to DC Voltage measurement
        const string SetMeasACV     = "CONF:VOLT:AC"       + "\n";     // Sets to AC Voltage measurement
        const string SetMeasDCI     = "CONF:CURR:DC"       + "\n";     // Sets to DC Voltage measurement
        const string SetMeasACI     = "CONF:CURR:AC"       + "\n";     // Sets to AC Voltage measurement
        const string SetMeasTemp    = "CONF:TEMP"          + "\n";     // Sets to Temperature measurement
        const string SetMeas2WRes   = "CONF:RES"           + "\n";     // Sets to 2-wire resistance measurement
        const string SetMeas4WRes   = "CONF:FRES"          + "\n";     // Sets to 4-wire resistance measurement
        const string SetMeasFreqV   = "CONF:FREQ:VOLTAGE"  + "\n";     // Sets to Voltage Frequency measurement
        const string SetMeasFreqC   = "CONF:FREG:CURR"     + "\n";     // Sets to Current Frequency measurement
        const string SetMeasDiode   = "CONF:DIOD"          + "\n";     // Sets to Diode measurement, 5V Range
        const string SetMeasCap     = "CONF:CAP"           + "\n";     // Sets to Diode measurement, 5V Range

       Point? prevPosition = null;
       ToolTip tooltip = new ToolTip();
       
         
        // ------------------------------------ FORM 1 -------------------------------------------------------------
        public Form1()
        {
            InitializeComponent();
            chart1.ChartAreas[0].AxisY.IsStartedFromZero = false;               // graph dont starts @ zero
            this.DoubleBuffered = true;
            initialZoomValueX = chart1.ChartAreas[0].AxisX.ScaleView.Size;     // Get Scaleview Size to hand over when switched back from Roll-Mode to Shrink-Mode
            initialZoomValueY = chart1.ChartAreas[0].AxisY.ScaleView.Size;
            Checkbox_Roll.SetBounds(this.Size.Width - 120, this.Size.Height - 80, 74, 17);
            trackBar1.SetBounds(this.Size.Width - 400, this.Size.Height - 80, 250, 25);
            zoom_factor = trackBar1.Maximum;
            MeasureMode = SetMeasDCV;

            chart1.ChartAreas[0].CursorX.IsUserEnabled = true;
            chart1.ChartAreas[0].CursorX.IsUserSelectionEnabled = true;
            chart1.ChartAreas[0].AxisX.ScaleView.Zoomable = true;
            chart1.ChartAreas[0].AxisX.ScrollBar.IsPositionedInside = true;

            chart1.ChartAreas[0].CursorY.IsUserEnabled = true;
            chart1.ChartAreas[0].CursorY.IsUserSelectionEnabled = true;
            chart1.ChartAreas[0].AxisY.ScaleView.Zoomable = true;
            chart1.ChartAreas[0].AxisY.ScrollBar.IsPositionedInside = true;

            

        }

        public static String GetTimestamp(DateTime value)  // Liefert einen Zeitstempel
        {
            return value.ToString("HH:mm:ss.fff");  // Zeitformat
        }

        private void buttonStart_Click(object sender, EventArgs e)
        {

            if (buttonStart.Text == "Disconnect")
            {
                if (serialPort1.IsOpen)
                {
                    DataReadTimer.Enabled = false;
                    DataAquisition = false;
                    serialPort1.Close();
                    Stop_Button.Enabled = false;
                    ReadHMCData_Button.Enabled = false;
                    Status_Text_Box.AppendText("Disconnected" + "\r\n");
                    textBox1.ReadOnly = true;
                }
                //ReadHMCData_Button.Enabled = false;
                buttonStart.Text = "Connect";
                Stop_Button.Checked = false;
                ReadHMCData_Button.Checked = false;
                return;
            }

            // MeasureMode = "";
            // Search of COM Port Devices in ManagementObjectCollection
            int Str_index;
            ManagementScope scope = new ManagementScope(@"\\" + Environment.MachineName + @"\root\CIMV2");
             
            SelectQuery sq = new SelectQuery("SELECT Name,Caption FROM Win32_PnPEntity WHERE Caption like '%HMC%'");  // Hier könnte man dann auch HMC eintragen
            ManagementObjectSearcher searcher = new ManagementObjectSearcher(scope, sq);
            ManagementObjectCollection moc = searcher.Get();

            foreach (ManagementObject mo in moc)
            {
                object propName = mo.Properties["Name"].Value;
                if (propName == null) { continue; }
              
                // Extraction of COM Port Number --> COMX
                Str_index = propName.ToString().IndexOf("COM");                 // Ermittelt die Stelle, bei der COM im String beginnt
                ComPort = (propName.ToString()).Substring(Str_index, 5);        // Bildet einen Substring der nur COM + 2 Zeichen enthält
                if ((ComPort.Substring(ComPort.Length - 1, 1)) == ")") ComPort = ComPort.Substring(0, ComPort.Length - 1);         // Prüfung ob COM-Port 1 oder 2 Stellen hat
                Status_Text_Box.AppendText((propName.ToString()).Substring(0, Str_index-1) + " found on ");    // Stellt den Device Namen dar
                Status_Text_Box.AppendText(ComPort + "\n");
            }
     
            // If a Comport is found and the a Comport is not open
            if ((!(ComPort == null))&(!serialPort1.IsOpen))
            {
                serialPort1.PortName = ComPort;
                serialPort1.BaudRate = 115200;
                serialPort1.ReadTimeout = 500;
                serialPort1.WriteTimeout = 1000;
                serialPort1.Open();
                System.Threading.Thread.Sleep(50);
            }
            else Status_Text_Box.AppendText("Device not found" + "\n");
           
            
            if (serialPort1.IsOpen)
            {
                //ReadHMCData_Button.Enabled = true;
                //Stop_Button.Enabled = false;
                textBox1.ReadOnly = false;
                read_HMCVer = true;
                serialPort1.Write("*IDN?"+"\n"); // Request ID Number
                Status_Text_Box.AppendText("Connecting...\n");
                System.Threading.Thread.Sleep(50);
                buttonStart.Text = "Disconnect";
            }
            else return;
        }

        private void Stop_Button_Click(object sender, EventArgs e)
        {
            if (serialPort1.IsOpen)
            {
                DataReadTimer.Enabled = false;
                DataAquisition = false;
                ReadHMCData_Button.Checked = false;
                Stop_Button.Checked = true;
                Status_Text_Box.AppendText("Stopped" + "\r\n");
                textBox1.ReadOnly = true;
            }
            else Status_Text_Box.AppendText("Disconnected" + "\r\n");
        }

        // Start Measurement
        private void ReadHMCData_Button_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) return;
            read_HMCVer = false;
            DataAquisition = true;
            answer = "";
            DataReadTimer.Enabled = true;                       // Read out starts with enabling DataReadTimer
            Status_Text_Box.AppendText("Reading...\n");         
        }

        // Read data from HMC8012 via FETCh
        private void DataReadTimer_Tick(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen)
            {
                Status_Text_Box.AppendText("Not connected!" + "\n");                                        // Write out in case that serial port is not open
                return;
            }
            try { serialPort1.Write("FETCh?" + "\n"); }                                                     // Request current displayed value from HMC8012
            catch
            {
                this.Invoke(new EventHandler(Stop_Button_Click));
                Status_Text_Box.AppendText("Device disconnected! ");                                        // For debugging purpose only
                buttonStart.Text = "Connect";
                Stop_Button.Checked = false;
                ReadHMCData_Button.Checked = false;
                System.Threading.Thread.Sleep(1500);
                //serialPort1.Close();
                //this.Invoke(new EventHandler(buttonStart_Click));   // Serial Port appears to be open but isnt, cant be closed
            }
        }

        // Clears the chart and text box content 
        private void Clear_Click(object sender, EventArgs e)     
        {
            chart1.Series[0].Points.Clear();                                                                // Clears chart points
            textBox1.Text="";                                                                               // Clears the textbox
            chartindex = 0;                                                                                 // Reset the chart index to start from zero again
            MaxDisplayed = 0;                                                                               // Resets displayed max value
            MinDisplayed = 0;                                                                               // Resets displayed mmin value
            d = 0;                                                                                          // Resets current calculated value
            AVG = 0;
            AVG_Sum = 0;

            MaxCalc.Text = MaxDisplayed.ToString("0.##0").Replace(",", ".") + MUnit;                        // display Max Hold value
            MinCalc.Text = MinDisplayed.ToString("0.##0").Replace(",", ".") + MUnit;                        // display Min Hold value
            Ausgabe1.Text = d.ToString("0.##0").Replace(",", ".") + MUnit + " ";                            // display Current value
            PeakToPeak.Text = (MaxDisplayed - MinDisplayed).ToString("0.##0").Replace(",", ".") + MUnit;    // Display Peak to Peak value
            Avrg_Value.Text = AVG.ToString("0.##0").Replace(",", ".") + MUnit;                              // Display Average Value

        }

        private void ConvertReceivedDataToDecimal()
        {
            chartindex++;                                                                                 // increases the chartindex
            String timeStamp = GetTimestamp(DateTime.Now);                                                  // creates actual time stamp
            textBox1.AppendText(chartindex.ToString("0000") + "; " + timeStamp + "; " + answer);            // Print out chartindex, timestamp and current value
                       
            try { d = Decimal.Parse(answer.Replace(".", ","), System.Globalization.NumberStyles.Float); }   // convert measured value from string to decimal
            catch
            {
                Status_Text_Box.AppendText("Catch! This value is fault: " + answer + " X" + "\n");          // For debugging purpose only
                this.Invoke(new EventHandler(Stop_Button_Click));                                           // Stops reading if occuring
            }
            if (roll) chart1.ChartAreas[0].AxisX.ScaleView.Position = chartindex - zoom_factor;             // Only for roll mode
            answer = "";  // Clear receiving buffer after conversion
         
        }

        private void DisplayMinMax()
        {
            if (chartindex == 1)                                                                            // Set initial value for min/max
            {
                min = d;
                max = d;
            }
            if (d > max) max = d;
            if (d < min) min = d;

            // Average Value calculation
            AVG_Sum = AVG_Sum + d;                                               // Sum single values 
            AVG = AVG_Sum / (chartindex);                                        // Calculate Average value
           

            // Set Unit Voltage and distinguish between mV and V
            if ((max < 1m) & ((MeasureMode == SetMeasACV) | (MeasureMode == SetMeasDCV)))
            {
                MUnit = " mV";
                MaxDisplayed = max * 1000;
                MinDisplayed = min * 1000;
                d = d * 1000;
            }
            else if ((max >= 1m) & ((MeasureMode == SetMeasACV) | (MeasureMode == SetMeasDCV)))
            {
                MUnit = " V";
                MaxDisplayed = max;
                MinDisplayed = min;
            }
            
            // Set Unit Ampere and distinguish between mA and A
            if ((max < 1m) & ((MeasureMode == SetMeasACI) | (MeasureMode == SetMeasDCI)))
            {
                MUnit = " mA";
                MaxDisplayed = max * 1000;
                MinDisplayed = min * 1000;
            }
           else if ((max >= 1m) & ((MeasureMode == SetMeasACI) | (MeasureMode == SetMeasDCI)))
            {
                MUnit = " A";
                MaxDisplayed = max;
                MinDisplayed = min;
            }
            
            // Set Unit Hertz and distinguish between Hz and kHz
            if ((max < 1000m) & ((MeasureMode == SetMeasFreqV) | (MeasureMode == SetMeasFreqC)))
            {
                MUnit = " Hz";
                MaxDisplayed = max;
                MinDisplayed = min;
            }
            else if ((max >= 1000m) & ((MeasureMode == SetMeasFreqV) | (MeasureMode == SetMeasFreqC)))
            {
                MUnit = " kHz";
                MaxDisplayed = max / 1000;      // calculates value to kHz
                MinDisplayed = min / 1000;      // calculates value to kHz
            }

            // Set Unit Ohm and distinguish between Ohm and kOhm
            if ((max < 1000m) & (MeasureMode == SetMeas2WRes)|(MeasureMode == SetMeas4WRes))
            {
                MUnit = " Ω";
                MaxDisplayed = max;
                MinDisplayed = min;
            }
            else if ((max >= 1000m) & (MeasureMode == SetMeas2WRes)|(MeasureMode == SetMeas4WRes))
            {
                MUnit = " kΩ";
                MaxDisplayed = max / 1000;
                MinDisplayed = min / 1000;
                AVG = AVG / 1000;
                //d = d / 1000;
            }

            // Set Unit Fahrad and distinguish between nF and µF
            if ((max < 0.000001m) & (MeasureMode == SetMeasCap))
            {
                MUnit = " nF";
                MaxDisplayed = max*1000000000;
                MinDisplayed = min*1000000000;
                d = d * 1000000000;
            }
            else if ((max >= 0.000001m) & (MeasureMode == SetMeasCap))
            {
                MUnit = " µF";
                MaxDisplayed = max * 1000000;
                MinDisplayed = min * 1000000;
                d = d * 1000000;
            }

            if (max < 1)
            {
                MaxCalc.Text = MaxDisplayed.ToString("0.##0").Replace(",", ".") + MUnit;                        // Max Hold
                MinCalc.Text = MinDisplayed.ToString("0.##0").Replace(",", ".") + MUnit;                        // Min Hold
                Ausgabe1.Text = d.ToString("0.##0").Replace(",", ".") + MUnit + " ";                            // Aktueller Wert
                PeakToPeak.Text = (MaxDisplayed - MinDisplayed).ToString("0.##0").Replace(",", ".") + MUnit;    // Peak to Peak
                Avrg_Value.Text = AVG.ToString("0.##0").Replace(",", ".") + MUnit;                              // Display Average Value

            }

            else if ((max > 10) & (max < 100))
            {
                MaxCalc.Text = MaxDisplayed.ToString("0.###0").Replace(",", ".") + MUnit;                     // Max Hold
                MinCalc.Text = MinDisplayed.ToString("0.###0").Replace(",", ".") + MUnit;                     // Min Hold
                Ausgabe1.Text = d.ToString("0.###0").Replace(",", ".") + MUnit + " ";                         // Aktueller Wert
                PeakToPeak.Text = (MaxDisplayed - MinDisplayed).ToString("0.###0").Replace(",", ".") + MUnit; // Peak to Peak
                Avrg_Value.Text = AVG.ToString("0.###0").Replace(",", ".") + MUnit;                           // Display Average Value
            }


            else if ((max > 1) & (max < 10))
            {
                MaxCalc.Text = MaxDisplayed.ToString("0.#####0").Replace(",", ".") + MUnit;                     // Max Hold
                MinCalc.Text = MinDisplayed.ToString("0.#####0").Replace(",", ".") + MUnit;                     // Min Hold
                Ausgabe1.Text = d.ToString("0.#####0").Replace(",", ".") + MUnit + " ";                         // Aktueller Wert
                PeakToPeak.Text = (MaxDisplayed - MinDisplayed).ToString("0.#####0").Replace(",", ".") + MUnit; // Peak to Peak
                Avrg_Value.Text = AVG.ToString("0.#####0").Replace(",", ".") + MUnit;                           // Display Average Value
            }

            else if ((max > 100) & (max < 1000))
            {
                MaxCalc.Text = MaxDisplayed.ToString("0.###0").Replace(",", ".") + MUnit;                     // Max Hold
                MinCalc.Text = MinDisplayed.ToString("0.###0").Replace(",", ".") + MUnit;                     // Min Hold
                Ausgabe1.Text = d.ToString("0.###0").Replace(",", ".") + MUnit + " ";                         // Aktueller Wert
                PeakToPeak.Text = (MaxDisplayed - MinDisplayed).ToString("0.###0").Replace(",", ".") + MUnit; // Peak to Peak
                Avrg_Value.Text = AVG.ToString("0.###0").Replace(",", ".") + MUnit;                           // Display Average Value
            }
            else if ((max > 1000) & (max < 10000))
            {
                MaxCalc.Text = MaxDisplayed.ToString("0.##0").Replace(",", ".") + MUnit;                     // Max Hold
                MinCalc.Text = MinDisplayed.ToString("0.##0").Replace(",", ".") + MUnit;                     // Min Hold
                Ausgabe1.Text = d.ToString("0.##0").Replace(",", ".") + MUnit + " ";                         // Aktueller Wert
                PeakToPeak.Text = (MaxDisplayed - MinDisplayed).ToString("0.##0").Replace(",", ".") + MUnit; // Peak to Peak
                Avrg_Value.Text = AVG.ToString("0.##0").Replace(",", ".") + MUnit;                           // Display Average Value
            }
            else if ((max > 10000) & (max < 100000))
            {
                MaxCalc.Text = MaxDisplayed.ToString("0.#0").Replace(",", ".") + MUnit;                     // Max Hold
                MinCalc.Text = MinDisplayed.ToString("0.#0").Replace(",", ".") + MUnit;                     // Min Hold
                Ausgabe1.Text = d.ToString("0.#0").Replace(",", ".") + MUnit + " ";                         // Aktueller Wert
                PeakToPeak.Text = (MaxDisplayed - MinDisplayed).ToString("0.#0").Replace(",", ".") + MUnit; // Peak to Peak
                Avrg_Value.Text = AVG.ToString("0.#0").Replace(",", ".") + MUnit;                           // Display Average Value
            }
            else
            {
                MaxCalc.Text = MaxDisplayed.ToString("0.0").Replace(",", ".") + MUnit;                     // Max Hold
                MinCalc.Text = MinDisplayed.ToString("0.0").Replace(",", ".") + MUnit;                     // Min Hold
                Ausgabe1.Text = d.ToString("0.0").Replace(",", ".") + MUnit + " ";                         // Aktueller Wert
                PeakToPeak.Text = (MaxDisplayed - MinDisplayed).ToString("0.0").Replace(",", ".") + MUnit; // Peak to Peak
                Avrg_Value.Text = AVG.ToString("0.0").Replace(",", ".") + MUnit;                           // Display Average Value
            }



            //else
            //{
            //    MaxCalc.Text = MaxDisplayed.ToString("0.##0").Replace(",", ".") + MUnit;                        // Max Hold
            //    MinCalc.Text = MinDisplayed.ToString("0.##0").Replace(",", ".") + MUnit;                        // Min Hold
            //    Ausgabe1.Text = d.ToString("0.##0").Replace(",", ".") + MUnit + " ";                            // Aktueller Wert
            //    PeakToPeak.Text = (MaxDisplayed - MinDisplayed).ToString("0.##0").Replace(",", ".") + MUnit;    // Peak to Peak
            //    Avrg_Value.Text = AVG.ToString("0.##0").Replace(",", ".") + MUnit;                              // Display Average Value
            //}


        }

        private void DisplayText(object sender, EventArgs e)
        {
            if (read_HMCVer)
            {
                if (answer.Contains("HMC8012"))
                {
                    serialPort1.Write("DISPlay:TEXT \"CONNECTED TO REMOTE PC\"" + "\n");        // Displaying Message on Device and setting Device to Remote                                  //serialPort1.Write("DISPlay:TEXT:CLE" + "\n");                         // Clearing Message
                    System.Threading.Thread.Sleep(1000);                                        // Wait to recognize Message on Device
                    serialPort1.Write("SYST:LOC" + "\n");
                    Status_Text_Box.AppendText(answer + " connected..." + "\n");
                    Status_Text_Box.AppendText("Please select measure mode..." + "\n");
                    ReadHMCData_Button.Enabled = true;
                    Stop_Button.Enabled = true;
                    Stop_Button.Checked = true;
                }
                else Status_Text_Box.AppendText("Device not connected" + "\n");
                read_HMCVer = false;
            }

            if (DataAquisition)
            {
                if (MeasureMode == SetMeasTemp)
                {
                    ConvertReceivedDataToDecimal();

                    // PT Sensor Resistance calculation
                    decimal r = (d / rn) - 1;
                    Temperature = (r * (255.8723m + r * (9.6m + (r * 0.878m))));

                    if (chartindex == 1) // Erster Wert für Min/Max Bildung
                    {
                        min = Temperature;
                        max = Temperature;
                    }
                    if (Temperature > max) max = Temperature;
                    if (Temperature < min) min = Temperature;

                    this.chart1.Series[0].Points.AddXY(chartindex, Temperature);

                    //answer = "";
                    MaxCalc.Text = max.ToString("0.##0").Replace(",", ".") + " [°C]";    // Max Hold
                    MinCalc.Text = min.ToString("0.##0").Replace(",", ".") + " [°C]";    // Min Hold
                                           
                    Ausgabe1.Text = Temperature.ToString("0.###0").Replace(",", ".") + " °C "; // Aktueller Wert
                }
                else
                {
                    ConvertReceivedDataToDecimal();
                    this.chart1.Series[0].Points.AddXY(chartindex, d);
                    DisplayMinMax();                                                     // Display Min/Max/Peak Values
                 }
             }
        }

        // Data received
        private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            answer = "";                                    // Clear string answer when start receiving
            while (!answer.Contains('\n'))                  // Continue until the line feed symbol is received
                {
                    answer += serialPort1.ReadExisting();   // merge single signs to string
                    if (!serialPort1.IsOpen)                // Check if serial port is still open
                    {
                        answer = "";                        // Avoid data fragments in case of disconnecting during receiving
                        return;
                    }
                }

                this.Invoke(new EventHandler(DisplayText)); // Start evaluation
        }
            
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (serialPort1.IsOpen) serialPort1.Close();    // Closes the Com-Port if Application is Closing
        }
      
        private void numericUpDown1_ValueChanged(object sender, EventArgs e)
        {
            DataReadTimer.Interval = Convert.ToInt32(numericUpDown1.Value);
        }

        private void Form1_Resize(object sender, EventArgs e)
        {
            //chart1.Size = new Size(chart1.Size.Width, this.Size.Height-250);
            chart1.Size = new Size(this.Size.Width, this.Size.Height - 250);
            Checkbox_Roll.SetBounds(this.Size.Width - 120, this.Size.Height - 80,74,17);
            trackBar1.SetBounds(this.Size.Width - 400, this.Size.Height - 80, 250, 25);
            Interval_Panel.SetBounds(50, this.Size.Height-80, 174, 36);
        }


 // ------------------------------------- BEGIN MEASUREMENT MODES --------------------------------------------------
        private void ChangeMeasurementMode()
        {
            DataReadTimer.Enabled = false;                          // Disable Data Read Timer
            answer = "";                                            // Clear Receiver Buffer
            this.Invoke(new EventHandler(Stop_Button_Click));       // Stop current measurement
            this.Invoke(new EventHandler(Clear_Click));             // Clear Chart Content
            //serialPort1.Write(MeasureMode);                         // Setting DC Current Measurement Mode
        }
        
         // DC Voltage Measurement Mode
         private void DCV_Button_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText("Not connected!" + "\n");
            else
            {
                MeasureMode = SetMeasDCV;           // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode
                serialPort1.Write(SetMeasDCV);      // Set Measurement Mode on Device
                Status_Text_Box.AppendText("Switched to DC Voltage measurement" + "\n");
                WireMode.Visible = false;           // Disable PT Temperature sensor visibility
                PTSensor.Visible = false;           // Disable Wire-Mode Option visibility
                chart1.Series[0].Name = "DC Voltage";
            }
        }

        // AC Voltage Measurement Mode
        private void ACV_Button_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText("Not connected!" + "\n");
            else
            {
                MeasureMode = SetMeasACV;           // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode
                serialPort1.Write(SetMeasACV);      // Set Measurement Mode on Device
                Status_Text_Box.AppendText("Switched to AC Voltage measurement" + "\n");
                WireMode.Visible = false;           // Disable PT Temperature sensor visibility
                PTSensor.Visible = false;           // Disable Wire-Mode Option visibility
                chart1.Series[0].Name = "AC Voltage";
            }
        }
        
        // DC Current Measurment Mode
        private void DCI_Button_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText("Not connected!" + "\n");
            else
            {
                MeasureMode = SetMeasDCI;           // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode
                serialPort1.Write(SetMeasDCI);      // Set Measurement Mode on Device
                Status_Text_Box.AppendText("Switched to DC Current measurement" + "\n");
                WireMode.Visible = false;           // Disable PT Temperature sensor visibility
                PTSensor.Visible = false;           // Disable Wire-Mode Option visibility
                chart1.Series[0].Name = "DC Current";
            }
        }

        

        // AC Current Measurment Mode
        private void ACI_Button_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText("Not connected!" + "\n");
            else
            {
                MeasureMode = SetMeasACI;           // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode
                serialPort1.Write(SetMeasDCI);      // Set Measurement Mode on Device
                Status_Text_Box.AppendText("Switched to AC Current measurement" + "\n");
                WireMode.Visible = false;           // Disable PT Temperature sensor visibility
                PTSensor.Visible = false;           // Disable Wire-Mode Option visibility
                chart1.Series[0].Name = "AC Current";
            }
        }

        



        // Voltage Frequency Measurment Mode
        private void FreqV_Button_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText("Not connected!" + "\n");
            else
            {
                MeasureMode = SetMeasFreqV;         // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode
                serialPort1.Write(SetMeasFreqV);    // Set Measurement Mode on Device
                Status_Text_Box.AppendText("Switched to Voltage Frequency measurement" + "\n");
                WireMode.Visible = false;           // Disable PT Temperature sensor visibility
                PTSensor.Visible = false;           // Disable Wire-Mode Option visibility
                chart1.Series[0].Name = "Voltage Frequency [Hz]";
            }
        }

        // Current Frequency Measurment Mode
        private void FreqI_Button_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText("Not connected!" + "\n");
            else
            {
                MeasureMode = SetMeasFreqC;         // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode
                serialPort1.Write(SetMeasFreqC);    // Set Measurement Mode on Device
                Status_Text_Box.AppendText("Switched to Current Frequency measurement" + "\n");
                WireMode.Visible = false;           // Disable PT Temperature sensor visibility
                PTSensor.Visible = false;           // Disable Wire-Mode Option visibility
                chart1.Series[0].Name = "Current Frequency [Hz]";
            }
        }
            
        // Diode Measurment Mode
        private void Diode_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText("Not connected!" + "\n");
            else
            {
                MeasureMode = SetMeasDiode;         // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode
                serialPort1.Write(SetMeasDiode);    // Set Measurement Mode on Device
                Status_Text_Box.AppendText("Switched to Diode measurement" + "\n");
                WireMode.Visible = false;           // Disable PT Temperature sensor visibility
                PTSensor.Visible = false;           // Disable Wire-Mode Option visibility
                chart1.Series[0].Name = "Diode Voltage [V]";
            }
        }

        // Resistance Measurment Mode
        private void R_Button_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText("Not connected!" + "\n");
            else
            {
                MeasureMode = SetMeas2WRes;         // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode
                serialPort1.Write(SetMeas2WRes);    // Set Measurement Mode on Device
                Status_Text_Box.AppendText("Switched to 2-Wire Resistance measurement" + "\n");
                PTSensor.Visible = false;           // Disable PT Temperature sensor visibility
                WireMode.Visible = true;            // Enables Wire-Mode Option visibility
                chart1.Series[0].Name = "Resistance [Ω]";
            }
        }

        // 2-Wire Resistance Measurement Mode (Default)
        private void TwoWire_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText("Not connected!" + "\n");
            else
            {
                MeasureMode = SetMeas2WRes;         // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode"
                serialPort1.Write(SetMeas2WRes);    // Set Measurement Mode on Device
                Status_Text_Box.AppendText("Switched to 2-Wire Resistance measurement" + "\n");
                chart1.Series[0].Name = "Resistance [Ω]";
            }
        }
        
        // 4-Wire Resistance Measurement Mode
        private void FourWire_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText("Not connected!" + "\n");
            else
            {
                MeasureMode = SetMeas4WRes;         // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode"
                serialPort1.Write(SetMeas4WRes);    // Set Measurement Mode on Device
                Status_Text_Box.AppendText("Switched to 4-Wire Resistance measurement" + "\n");
                chart1.Series[0].Name = "Resistance [Ω]";
            }
        }

        // Capacitance Measurement Mode
        private void Cap_Button_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText("Not connected!" + "\n");
            else
            {
                MeasureMode = SetMeasCap;           // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode"
                serialPort1.Write(SetMeasCap);      // Set Measurement Mode on Device
                Status_Text_Box.AppendText("Switched to Capacitance measurement" + "\n");
                PTSensor.Visible = false;           // Disable PT Temperature sensor visibility
                WireMode.Visible = false;           // Disable Wire-Mode Option visibility
                chart1.Series[0].Name = "Capacitance [µF]";
            }
        }
                                
        // Temperature Measurement Mode
        private void Temp_Button_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText("Not connected!" + "\n");
            else
            {
                MeasureMode = SetMeasTemp;          // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode"
                serialPort1.Write(SetMeasTemp);     // Set Measurement Mode on Device
                Status_Text_Box.AppendText("Switched to Temperatue measurement" + "\n");
                PTSensor.Visible = true;            // Enables PT Temperature sensor visibility
                WireMode.Visible = true;            // Enables Wire-Mode Option visibility
                chart1.Series[0].Name = "Temperature [°C]";
            }
        }

        // Select PT100 Temperature Sensor
        private void PT100_Click(object sender, EventArgs e)
        {
            rn = 100;   // 100 Ohm @ 0°C  
        }

        // Select PT500 Temperature Sensor
        private void PT500_Click(object sender, EventArgs e)
        {
            rn = 500;   // 500 Ohm @ 0°C 
        }

        // Select PT500 Temperature Sensor
        private void PT1000_Click(object sender, EventArgs e)
        {
            rn = 1000;  // 1000 Ohm @ 0°C
        }

// ------------------------------------- END MEASUREMENT MODES --------------------------------------------------
 
// ---------------------------------------- GRAPH CONTROLS ------------------------------------------------------            
                   
        private void trackBar1_Scroll(object sender, EventArgs e)
        {
            if (chartindex > trackBar1.Maximum) trackBar1.Maximum = 300;
            else trackBar1.Maximum = chartindex;

            zoom_factor = trackBar1.Value;
            if (Checkbox_Roll.Checked)
            {
                roll = true;
                chart1.ChartAreas[0].AxisX.ScaleView.Size = trackBar1.Value;        // Nur für den Roll-Modus
            }
            else
            {
                roll = false;
                chart1.ChartAreas[0].AxisX.ScaleView.Size = initialZoomValueX;       // NaN
            }
        }

        private void Checkbox_Roll_CheckedChanged(object sender, EventArgs e)
        {
            zoom_factor = trackBar1.Value;
            chart1.ChartAreas[0].AxisY.ScaleView.Size = initialZoomValueY;         // Auto
            if (Checkbox_Roll.Checked)
            {
                roll = true;
                if (chartindex > trackBar1.Maximum)
                {
                    chart1.ChartAreas[0].AxisX.ScaleView.Size = zoom_factor;//trackBar1.Maximum;        // Nur für den Roll-Modus
                    chart1.ChartAreas[0].AxisX.ScaleView.Position = chartindex - zoom_factor; // Nur für Roll-Modus
                }
                else
                {
                    chart1.ChartAreas[0].AxisX.ScaleView.Size = zoom_factor;
                    chart1.ChartAreas[0].AxisX.ScaleView.Position = zoom_factor-chartindex;
                }
            }
            else
            {
                roll = false;
                chart1.ChartAreas[0].AxisX.ScaleView.Size = initialZoomValueX;         // NaN
            }
        }

        private void chart1_MouseClick(object sender, MouseEventArgs e)
        {
            switch (e.Button)
            {
                    case MouseButtons.Left:
                    // Left click
                    break;

                    case MouseButtons.Right:
                    // Right click
                    chart1.ChartAreas[0].AxisY.ScaleView.Size = initialZoomValueY;         // Auto
                    chart1.ChartAreas[0].AxisX.ScaleView.Size = initialZoomValueX;         // NaN
                    break;
            }            
        }

        private void chart1_MouseMove(object sender, MouseEventArgs e)
        {
            var pos = e.Location;
            if (prevPosition.HasValue && pos == prevPosition.Value)
                return;
            tooltip.RemoveAll();
            prevPosition = pos;
            var results = chart1.HitTest(pos.X, pos.Y, false, ChartElementType.DataPoint);
            foreach (var result in results)
            {
                if (result.ChartElementType == ChartElementType.DataPoint)
                {
                    var prop = result.Object as DataPoint;
                    if (prop != null)
                    {
                        var pointXPixel = result.ChartArea.AxisX.ValueToPixelPosition(prop.XValue);
                        var pointYPixel = result.ChartArea.AxisY.ValueToPixelPosition(prop.YValues[0]);

                        // check if the cursor is really close to the point (5 pixels around the point)
                        if (Math.Abs(pos.X - pointXPixel) < 10 &&
                            Math.Abs(pos.Y - pointYPixel) < 10)
                        {
                            tooltip.Show("X=" + prop.XValue + ", Y=" + prop.YValues[0].ToString("0.#####0") + " " + MUnit, this.chart1,
                                            pos.X, pos.Y - 15);
                        }
                    }
                }
            }

        }   


    }
}
