﻿using System;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Management;
using System.IO.Ports;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

/* Next Steps:
- improve program structure
- improve stability during readout < 120ms 
- improve reconnection behaviour when device was switched of during readout
- improve readout speed and timestamp accuracy with following tasks
    - implement TMC VISA connection
    - implement high preceision timer
- implement CSV Data export
- implement speed test on system to avoid crash on to high read out speed 
- improve tooltip appearance
*/

namespace HMC8012Remote
{
    public partial class Form1 : Form
    {
        const string    ProgramVersion = "0.1";                                             // Program version appears in headline of main window
        static Timer    myTimer = new Timer();                                              // New Timer for Tick
        int             ChartIndex;                                                         // Index of received value 
        bool            Read_HMCVer =false, DataAquisition=false, PreUnit=false, WritingSucceed=true;
        string          HMCAnswer = "", ComPort;
        decimal         Min, Max, MinDisplayed, MaxDisplayed, Temperature, Rnull=1000, HMCDec, AVGSum, AVG;
        
        //------------------------------ Graph ----------------------------------
        double          InitialZoomValueX, InitialZoomValueY;
        Point?          prevPosition        = null;
        ToolTip         tooltip             = new ToolTip();
        bool            RollChart           = false;
        int             ZoomFactor;
        
        //-------------------------- Measure Modes ------------------------------
        string MeasureMode, MUnit, ActiveValue, PreUnitType;

        //------------------------ HMC8012 Commands -----------------------------
        const string DispMsg        = "DISP:TEXT";                                          // Text Message must be added like this ""Verbunden"" + "\n"
        const string DispTextClear  = "DISP:TEXT:CLE?\n";                                   // Clears text message in display

        // Meas requests a new measurement from Device, this takes apprx. 330 ms, faster is to require results with FETCh? Command, This gets value from Display 

        const string ReadID         = "*IDN?\n";
        const string ReadFetch      = "FETCh?\n";
        const string AllowLocalCtrl = "SYST:LOC\n";                                         // Enables local control of device    
        const string SetMeasDCV     = "CONF:VOLT:DC\n";                                     // Sets to DC Voltage measurement
        const string SetMeasACV     = "CONF:VOLT:AC\n";                                     // Sets to AC Voltage measurement
        const string SetMeasDCI     = "CONF:CURR:DC\n";                                     // Sets to DC Voltage measurement
        const string SetMeasACI     = "CONF:CURR:AC\n";                                     // Sets to AC Voltage measurement
        const string SetMeasTemp    = "CONF:TEMP\n";                                        // Sets to Temperature measurement
        const string SetMeas2WRes   = "CONF:RES\n";                                         // Sets to 2-wire resistance measurement
        const string SetMeas4WRes   = "CONF:FRES\n";                                        // Sets to 4-wire resistance measurement
        const string SetMeasFreqV   = "CONF:FREQ:VOLTAGE\n";                                // Sets to Voltage Frequency measurement
        const string SetMeasFreqC   = "CONF:FREG:CURR\n";                                   // Sets to Current Frequency measurement
        const string SetMeasDiode   = "CONF:DIODe\n";                                       // Sets to Diode measurement, 5V Range
        const string SetMeasCap     = "CONF:CAP\n";                                         // Sets to Diode measurement, 5V Range

        const string SetADCSlow     = "SENS:ADCR SLOW\n";                                   // Sets ADC conversion speed to Slow
        const string SetADCMed      = "SENS:ADCR MED\n";                                    // Sets ADC conversion speed to Medium
        const string SetADCFast     = "SENS:ADCR FAST\n";                                   // Sets ADC conversion speed to Fast
        const string SenseADC       = "SENS:ADCR?\n";                                       // Read ADC speed Setting

        // ------------------------------------ MESSAGES -----------------------------------------------------------

        const string NotCon         = "HMC8012 is not connected...\n";

        // ------------------------------------ FORM 1 -------------------------------------------------------------
        public Form1()
        {
            InitializeComponent();
            this.Text = "HAMEG HMC8012 Remote VCP " + "V"+ ProgramVersion;                  // Display Name and Version of Program in form headline
            // this.DoubleBuffered = true;                                                                    
            
            // Chart inits
            chart1.ChartAreas[0].CursorX.IsUserEnabled              = true;                 // Enables X Cursor for zoom 
            chart1.ChartAreas[0].CursorY.IsUserEnabled              = true;                 // Enables Y Cursor for zoom
            chart1.ChartAreas[0].CursorX.IsUserSelectionEnabled     = true;                 // Enables Range selection X-axis
            chart1.ChartAreas[0].CursorY.IsUserSelectionEnabled     = true;                 // Enables Range selection Y-axis
            chart1.ChartAreas[0].AxisX.ScaleView.Zoomable           = true;                 // Enables Zooming X-axis
            chart1.ChartAreas[0].AxisY.ScaleView.Zoomable           = true;                 // Enables Zooming Y-axis
            chart1.ChartAreas[0].AxisX.ScrollBar.IsPositionedInside = true;                 // Sets scrollbar inside the chart area
            chart1.ChartAreas[0].AxisY.ScrollBar.IsPositionedInside = true;                 // Sets scrollbar inside the chart area
            chart1.ChartAreas[0].AxisY.IsStartedFromZero            = false;                // graph dont starts @ zero// Necessary for autos cale Y-Axis
            InitialZoomValueX = chart1.ChartAreas[0].AxisX.ScaleView.Size;                  // Get Scaleview Size to hand over when switched back from Roll-Mode to Shrink-Mode
            InitialZoomValueY = chart1.ChartAreas[0].AxisY.ScaleView.Size;

            Checkbox_Roll.SetBounds(this.Size.Width - 120, this.Size.Height - 80, 74, 17);  // Set initial position for Roll control checkbox
            trackBar1.SetBounds(this.Size.Width - 400, this.Size.Height - 80, 250, 25);     // Set initial position for Roll control trackbar
            ZoomFactor = trackBar1.Maximum;                                                 // Set initial value for the zoomfactor
            textBox1.ReadOnly = true;
            MeasureMode = SetMeasDCV;                                                       // Set initial measure mode - necessary?
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (serialPort1.IsOpen) serialPort1.Close();                                    // Closes the Com-Port if Application is Closing
        }

        // Time stamp with standard timer // should be changed to high preceision timer
        public static String GetTimestamp(DateTime value)  
        {
            return value.ToString("HH:mm:ss.fff");                                          // Returns time stamp
        }

        private void buttonStart_Click(object sender, EventArgs e)
        {
            if (buttonStart.Text == "Disconnect")
            {
                if (serialPort1.IsOpen)
                {
                    DataReadTimer.Enabled       = false;
                    DataAquisition              = false;
                    serialPort1.Close();
                    Stop_Button.Enabled         = false;
                    ReadHMCData_Button.Enabled  = false;
                    Status_Text_Box.AppendText("Disconnected" + "\r\n");
                }
                //ReadHMCData_Button.Enabled = false;
                buttonStart.Text                = "Connect";
                Stop_Button.Checked             = false;
                ReadHMCData_Button.Checked      = false;
                return;
            }

            //MeasureMode = "";
            // Search of COM Port Devices in ManagementObjectCollection
            int Str_index;
            ManagementScope scope = new ManagementScope(@"\\" + Environment.MachineName + @"\root\CIMV2");
             
            SelectQuery sq = new SelectQuery("SELECT Name,Caption FROM Win32_PnPEntity WHERE Caption like '%HMC%'");  // Hier könnte man dann auch HMC eintragen
            ManagementObjectSearcher searcher = new ManagementObjectSearcher(scope, sq);
            ManagementObjectCollection moc = searcher.Get();

            foreach (ManagementObject mo in moc)
            {
                object propName = mo.Properties["Name"].Value;
                if (propName == null) { continue; }
              
                // Extraction of COM Port Number --> COMX
                Str_index = propName.ToString().IndexOf("COM");                 // Ermittelt die Stelle, bei der COM im String beginnt
                ComPort = (propName.ToString()).Substring(Str_index, 5);        // Bildet einen Substring der nur COM + 2 Zeichen enthält
                if ((ComPort.Substring(ComPort.Length - 1, 1)) == ")") ComPort = ComPort.Substring(0, ComPort.Length - 1);         // Prüfung ob COM-Port 1 oder 2 Stellen hat
                Status_Text_Box.AppendText((propName.ToString()).Substring(0, Str_index-1) + " found on ");    // Stellt den Device Namen dar
                Status_Text_Box.AppendText(ComPort + "\n");
            }
     
            // If a Com Port is found and the a Com Port is not open, init Com Port and open
            if ((!(ComPort == null))&(!serialPort1.IsOpen))
            {
                serialPort1.PortName = ComPort;
                serialPort1.BaudRate = 115200;
                serialPort1.ReadTimeout = 500;
                serialPort1.WriteTimeout = 1000;
                serialPort1.Open();
                System.Threading.Thread.Sleep(50);
            }
            else Status_Text_Box.AppendText("Device not found!" + "\n");
           
            
            if (serialPort1.IsOpen)
            {
                Read_HMCVer = true;
                try { serialPort1.Write(ReadID); } // Request ID Number
                catch
                {
                    Status_Text_Box.AppendText(NotCon);                                        // Write out in case that serial port is not open
                    return;
                }
                Status_Text_Box.AppendText("Connecting...\n");
                System.Threading.Thread.Sleep(50);
                buttonStart.Text = "Disconnect";
            }
            else return;
        }

        private void WriteHMCCommand(string Request)
        {
            try
            {
                serialPort1.Write(Request);
                WritingSucceed = true;
            }
            catch
            {
                Status_Text_Box.AppendText(NotCon);                                        // Write out in case that serial port is not open
                WritingSucceed = false;
            }
        }

        private void Stop_Button_Click(object sender, EventArgs e)
        {
            if (serialPort1.IsOpen)
            {
                DataReadTimer.Enabled       = false;
                DataAquisition              = false;
                ReadHMCData_Button.Checked  = false;
                Stop_Button.Checked         = true;
                Status_Text_Box.AppendText("Stopped" + "\r\n");
            }
            else Status_Text_Box.AppendText("Disconnected" + "\r\n");
        }

        // Start Measurement
        private void ReadHMCData_Button_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) return;
            Read_HMCVer                     = false;
            DataAquisition                  = true;
            HMCAnswer                       = "";
            DataReadTimer.Enabled           = true;                                                         // Read out starts with enabling DataReadTimer
            Status_Text_Box.AppendText("Reading...\n");         
        }

        // Read data from HMC8012 via FETCh
        private void DataReadTimer_Tick(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen)
            {
                Status_Text_Box.AppendText(NotCon);                                                         // Write out in case that serial port is not open
                return;
            }
            try { serialPort1.Write(ReadFetch); }                                                           // Request current displayed value from HMC8012
            catch
            {
                this.Invoke(new EventHandler(Stop_Button_Click));
                Status_Text_Box.AppendText("Device disconnected! ");                                        // For debugging purpose only
                buttonStart.Text = "Connect";
                Stop_Button.Checked         = false;
                ReadHMCData_Button.Checked  = false;
                System.Threading.Thread.Sleep(1500);
                this.Invoke(new EventHandler(buttonStart_Click));                                           // Serial Port appears to be open but isnt, cant be closed
            }
        }

        // Clears the chart and text box content 
        private void Clear_Click(object sender, EventArgs e)     
        {
            chart1.Series[0].Points.Clear();                                                                // Clears chart points
            textBox1.Text       ="";                                                                        // Clears the textbox
            ChartIndex          = 0;                                                                        // Reset the chart index to start from zero again
            MaxDisplayed        = 0;                                                                        // Resets displayed max value
            MinDisplayed        = 0;                                                                        // Resets displayed mmin value
            HMCDec              = 0;                                                                        // Resets current calculated value
            AVG                 = 0;
            AVGSum              = 0;

            MaxCalc.Text        = MaxDisplayed.ToString("0.##0").Replace(",", ".") + MUnit;                 // display Max Hold value
            MinCalc.Text        = MinDisplayed.ToString("0.##0").Replace(",", ".") + MUnit;                 // display Min Hold value
            Ausgabe1.Text       = HMCDec.ToString("0.##0").Replace(",", ".") + MUnit + " ";                 // display Current value
            PeakToPeak.Text     = (MaxDisplayed - MinDisplayed).ToString("0.##0").Replace(",", ".") + MUnit;// Display Peak to Peak value
            Avrg_Value.Text     = AVG.ToString("0.##0").Replace(",", ".") + MUnit;                          // Display Average Value
        }

        private void ConvertReceivedDataToDecimal()
        {
            ChartIndex++;                                                                                   // increases the chartindex
            String timeStamp = GetTimestamp(DateTime.Now);                                                  // creates actual time stamp
            textBox1.AppendText(ChartIndex.ToString("0000") + "; " + timeStamp + "; " + HMCAnswer.Replace(".",","));    // Print out chartindex, timestamp and current value
                       
            try { HMCDec = Decimal.Parse(HMCAnswer.Replace(".", ","), System.Globalization.NumberStyles.Float); }   // convert measured value from string to decimal
            catch
            {
                Status_Text_Box.AppendText("Catch! This value is fault: " + HMCAnswer + " X" + "\n");       // For debugging purpose only
                this.Invoke(new EventHandler(Stop_Button_Click));                                           // Stops reading if occuring
            }
            if (RollChart) chart1.ChartAreas[0].AxisX.ScaleView.Position = ChartIndex - ZoomFactor;         // Only for roll mode
            HMCAnswer = "";                                                                                 // Clear receiving buffer after conversion
        }

        private void Evaluation()
        {
            if (ChartIndex == 1)                                                                            // Set initial value for min/max
            {
                Min = HMCDec;
                Max = HMCDec;
            }
            if (HMCDec > Max) Max = HMCDec;
            if (HMCDec < Min) Min = HMCDec;

            // Average Value calculation
            AVGSum = AVGSum + HMCDec;                                                                       // Sum single values 
            AVG = AVGSum / (ChartIndex);                                                                    // Calculate Average value

            // Auswertung Spannung und Strom
            if ((MeasureMode == SetMeasACV) | (MeasureMode == SetMeasDCV) | (MeasureMode == SetMeasACI) | (MeasureMode == SetMeasDCI) | (MeasureMode == SetMeasDiode))
            {
                PreUnitType         = "m";
                
                if (HMCDec< 0.4m)
                {
                    PreUnit         = true;
                    HMCDec          = HMCDec * 1000;
                    ActiveValue     = HMCDec.ToString("0.##0").Replace(",", ".");
                }

                else if ((HMCDec > 0.4m) & (HMCDec < 4m))
                {
                    PreUnit         = false;
                    ActiveValue     = HMCDec.ToString("0.### #0").Replace(",", ".");
                }
                else if ((HMCDec > 4m) & (HMCDec < 40m))
                {
                    PreUnit         = false;
                    ActiveValue     = HMCDec.ToString("0.### 0").Replace(",", ".");
                }
                else if ((HMCDec > 40m) & (HMCDec < 400m))
                {
                    PreUnit         = false;
                    ActiveValue     = HMCDec.ToString("0.##0").Replace(",", ".");
                }

            // Ausgabe mV, V, mA, A
                if (PreUnit)
                {
                    MaxDisplayed    = Max * 1000;
                    MinDisplayed    = Min * 1000;
                    AVG             = AVG * 1000;
                    Ausgabe1.Text   = ActiveValue + " m" + MUnit + " ";                                                                 // Aktueller Wert 
                    MaxCalc.Text    = MaxDisplayed.ToString("0.##0").Replace(",", ".") + " "+ PreUnitType + MUnit;                      // Max Hold
                    MinCalc.Text    = MinDisplayed.ToString("0.##0").Replace(",", ".") + " "+ PreUnitType + MUnit;                      // Min Hold
                    PeakToPeak.Text = (MaxDisplayed - MinDisplayed).ToString("0.##0").Replace(",", ".") + " " + PreUnitType + MUnit;    // Peak to Peak
                    Avrg_Value.Text = AVG.ToString("0.##0").Replace(",", ".") + " "+ PreUnitType + MUnit;                               // Display Average Value
                }
                else
                {
                    MaxDisplayed    = Max;
                    MinDisplayed    = Min;
                    Ausgabe1.Text   = ActiveValue + " " + MUnit + " ";                                                                  // Aktueller Wert 
                    MaxCalc.Text    = MaxDisplayed.ToString("0.### #0").Replace(",", ".") + " " + MUnit;                                // Max Hold
                    MinCalc.Text    = MinDisplayed.ToString("0.### #0").Replace(",", ".") + " " + MUnit;                                // Min Hold
                    PeakToPeak.Text = (MaxDisplayed - MinDisplayed).ToString("0.### #0").Replace(",", ".") + " " + MUnit;               // Peak to Peak
                    Avrg_Value.Text = AVG.ToString("0.### #0").Replace(",", ".") + " " + MUnit;                                         // Display Average Value
                }
            }
            
            // Following part ensures that the displayed digits are shown like on HMC8012 in ADC fast mode - hystersis not implemented so far
            if ((MeasureMode == SetMeas2WRes) | (MeasureMode == SetMeas4WRes) | (MeasureMode == SetMeasFreqV) | (MeasureMode == SetMeasFreqC))
            {
                
                PreUnitType = "k";
                if (HMCDec < 40m)
                {
                    PreUnit = false;
                     ActiveValue    = HMCDec.ToString("0.###0").Replace(",", ".");
                }
                else if ((HMCDec > 40m) & (HMCDec < 400m))
                {
                    PreUnit = false;
                    ActiveValue     = HMCDec.ToString("0.##0").Replace(",", ".");
                }
                else if ((HMCDec > 400m) & (HMCDec < 4000m))
                {
                    PreUnit         = true;
                    HMCDec          = HMCDec / 1000;
                    ActiveValue     = HMCDec.ToString("0.####0").Replace(",", ".");
                }
                else if ((HMCDec > 4000m) & (HMCDec < 40000m))
                {
                    PreUnit         = true;
                    HMCDec          = HMCDec / 1000;
                    ActiveValue     = HMCDec.ToString("0.###0").Replace(",", ".");
                }
                else if ((HMCDec > 40000m) & (HMCDec < 40000m))
                {
                    PreUnit         = true;
                    HMCDec          = HMCDec / 1000;
                    ActiveValue     = HMCDec.ToString("0.##0").Replace(",", ".");
                }
                
                // Ausgabe Ohm oder Hz
                if (PreUnit)
                {
                    MaxDisplayed    = Max/1000;
                    MinDisplayed    = Min/1000;
                    AVG             = AVG / 1000;
                    Ausgabe1.Text   = ActiveValue + " " + PreUnitType + MUnit + " ";                                                        // Aktueller Wert 
                    MaxCalc.Text    = MaxDisplayed.ToString("0.####0").Replace(",", ".") + " " + PreUnitType + MUnit;                       // Max Hold
                    MinCalc.Text    = MinDisplayed.ToString("0.####0").Replace(",", ".") + " " + PreUnitType + MUnit;                       // Min Hold
                    PeakToPeak.Text = (MaxDisplayed - MinDisplayed).ToString("0.####0").Replace(",", ".") + " " + PreUnitType + MUnit;      // Peak to Peak
                    Avrg_Value.Text = AVG.ToString("0.####0").Replace(",", ".") + " " + PreUnitType + MUnit;                                // Display Average Value
                }
                else
                {
                    MaxDisplayed    = Max; 
                    MinDisplayed    = Min; 
                    Ausgabe1.Text   = ActiveValue + " " + MUnit + " ";                                                                      // Aktueller Wert 
                    MaxCalc.Text    = MaxDisplayed.ToString("0.###0").Replace(",", ".") + " " + MUnit;                                      // Max Hold
                    MinCalc.Text    = MinDisplayed.ToString("0.###0").Replace(",", ".") + " " + MUnit;                                      // Min Hold
                    PeakToPeak.Text = (MaxDisplayed - MinDisplayed).ToString("0.###0").Replace(",", ".") + " " + MUnit;                     // Peak to Peak
                    Avrg_Value.Text = AVG.ToString("0.###0").Replace(",", ".") + " " + MUnit;                                               // Display Average Value
                }
            }
            
            // Evaluation Capacity - only two ranges are distinguished
            if (MeasureMode == SetMeasCap)
            {
                if (HMCDec < 0.000001m)
                {
                    MUnit           = " nF";                                                                                // Set Unit to "nF"
                    MaxDisplayed    = Max       * 1000000000;                                                               // Recalculate for display of nF value
                    MinDisplayed    = Min       * 1000000000;                                                               // Recalculate for display of nF value
                    HMCDec          = HMCDec    * 1000000000;                                                               // Recalculate for display of nF value
                    AVG             = AVG       * 1000000000;                                                               // Recalculate for display of nF value
                }
                else if (HMCDec > 0.000001m)
                {
                    MUnit           = " µF";                                                                                // Set Unit to "µF"
                    MaxDisplayed    = Max       * 1000000;                                                                  // Recalculate for display of µF value
                    MinDisplayed    = Min       * 1000000;                                                                  // Recalculate for display of µF value
                    HMCDec          = HMCDec    * 1000000;                                                                  // Recalculate for display of µF value
                    AVG             = AVG       * 1000000;                                                                  // Recalculate for display of µF value
                }
                ActiveValue         = HMCDec.ToString("0.##0").Replace(",", ".");
                Ausgabe1.Text       = ActiveValue + " " + MUnit + " ";                                                      // Print current value 
                MaxCalc.Text        = MaxDisplayed.ToString("0.##0").Replace(",", ".") + " " + MUnit;                       // Print Max Hold value
                MinCalc.Text        = MinDisplayed.ToString("0.##0").Replace(",", ".") + " " + MUnit;                       // Print Min Hold value
                PeakToPeak.Text     = (MaxDisplayed - MinDisplayed).ToString("0.##0").Replace(",", ".") + " " + MUnit;      // Print Peak to Peak value
                Avrg_Value.Text     = AVG.ToString("0.##0").Replace(",", ".") + " " + MUnit;                                // Print average value
            }
            
            // Calculate and output Temperature mode
            if (MeasureMode == SetMeasTemp)
                {
                    ActiveValue     = HMCDec.ToString("0.0").Replace(",", ".");
                    MaxDisplayed    = Max;
                    MinDisplayed    = Min;
                    Ausgabe1.Text   = ActiveValue + " " + MUnit + " ";                                                      // Print current value  
                    MaxCalc.Text    = MaxDisplayed.ToString("0.0").Replace(",", ".") + " " + MUnit;                         // Print Max Hold value
                    MinCalc.Text    = MinDisplayed.ToString("0.0").Replace(",", ".") + " " + MUnit;                         // Print Min Hold value
                    PeakToPeak.Text = (MaxDisplayed - MinDisplayed).ToString("0.0").Replace(",", ".") + " " + MUnit;        // Print Peak to Peak value
                    Avrg_Value.Text = AVG.ToString("0.0").Replace(",", ".") + " " + MUnit;                                  // Print average value
            }
        }

        private void CalcTempFromRes()
        {
            //if (MeasureMode == SetMeasTemp)
            //{
            //    ConvertReceivedDataToDecimal();
            //    // PT Sensor Resistance calculation
            //    decimal r = (d / Rnull) - 1;
            //    Temperature = (r * (255.8723m + r * (9.6m + (r * 0.878m))));

            //    if (chartindex == 1) // Erster Wert für Min/Max Bildung
            //    {
            //        min = Temperature;
            //        max = Temperature;
            //    }
            //    if (Temperature > max) max = Temperature;
            //    if (Temperature < min) min = Temperature;

            //    // Average Value calculation
            //    AVG_Sum = AVG_Sum + Temperature;                                              // Sum single values 
            //    AVG = AVG_Sum / (chartindex);                                                 // Calculate Average value

            //    this.chart1.Series[0].Points.AddXY(chartindex, Temperature);

            //    //answer = "";
            //    MaxCalc.Text = max.ToString("0.##0").Replace(",", ".") + MUnit;               // Max Hold
            //    MinCalc.Text = min.ToString("0.##0").Replace(",", ".") + MUnit;               // Min Hold
            //    PeakToPeak.Text = (max - min).ToString("0.###0").Replace(",", ".") + MUnit;   // Peak to Peak Value
            //    Ausgabe1.Text = Temperature.ToString("0.###0").Replace(",", ".") + MUnit;     // Current Value
            //    Avrg_Value.Text = AVG.ToString("0.##0").Replace(",", ".") + MUnit;            // Average Value
            //}
        }

        private void ProcessRX(object sender, EventArgs e)
        {
            // Check for ADC Speed
            if (HMCAnswer == "SLOW\n")
            {
                Radio_Button_ADCSlow.Checked = true;
            }
            if (HMCAnswer == "MED\n")
            {
                Radio_Button_ADCMed.Checked = true;
            }
            if (HMCAnswer == "FAST\n")
            {
                Radio_Button_ADCFast.Checked = true;
            }
            // --------------------------------
            // Readout HMC8012 Version values
            if (Read_HMCVer)
            {
                if (HMCAnswer.Contains("HMC8012"))
                {
                    WriteHMCCommand("DISPlay:TEXT \"CONNECTED TO REMOTE PC\"" + "\n");
                    //serialPort1.Write("DISPlay:TEXT \"CONNECTED TO REMOTE PC\"" + "\n");        // Displaying Message on Device and setting Device to Remote                                  //serialPort1.Write("DISPlay:TEXT:CLE" + "\n");                         // Clearing Message
                    System.Threading.Thread.Sleep(1000);                                        // Wait to recognize Message on Device
                    WriteHMCCommand(AllowLocalCtrl);
                    //serialPort1.Write(AllowLocalCtrl);                                          // Allows local device control
                    System.Threading.Thread.Sleep(100);
                    WriteHMCCommand(SenseADC);
                    //serialPort1.Write(SenseADC);                                                // Read ADC speed setting
                    Status_Text_Box.AppendText(HMCAnswer + " connected..." + "\n");
                    Status_Text_Box.AppendText("Please select measure mode..." + "\n");
                    ReadHMCData_Button.Enabled  = true;
                    Stop_Button.Enabled         = true;
                    Stop_Button.Checked         = true;
                }
                else Status_Text_Box.AppendText(NotCon);
                Read_HMCVer = false;
            }
            if (DataAquisition)
            {
                ConvertReceivedDataToDecimal();
                this.chart1.Series[0].Points.AddXY(ChartIndex, HMCDec);
                Evaluation();                                                                   // Display Min/Max/Peak Values
            }
        }

        // Data received function
        private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            HMCAnswer = "";                                                                     // Clear string answer when start receiving
            while (!HMCAnswer.Contains('\n'))                                                   // Continue until the line feed symbol is received
            {
                HMCAnswer += serialPort1.ReadExisting();                                        // merge single signs to string
                if (!serialPort1.IsOpen)                                                        // Check if serial port is still open
                {
                    HMCAnswer = "";                                                             // Avoid data fragments in case of disconnecting during receiving
                    return;
                }
            }
            this.Invoke(new EventHandler(ProcessRX));                                           // Start evaluation
         }
            
         // Adapt object positions in case of resizing window
        private void Form1_Resize(object sender, EventArgs e)
        {
            chart1.Size = new Size(this.Size.Width, this.Size.Height - 250);                    // Adapts position and Size of Chart area
            Checkbox_Roll.SetBounds(this.Size.Width - 120, this.Size.Height - 80,74,17);        // Adapts Checkbox_Roll position
            trackBar1.SetBounds(this.Size.Width - 400, this.Size.Height - 80, 250, 25);         // Adapts Trackbar1 position
        }
        
 // ------------------------------------- BEGIN MEASUREMENT SETTINGS ------------------------------------------------
        private void ChangeMeasurementMode()
        {
            DataReadTimer.Enabled   = false;                                                    // Disable Data Read Timer
            AVG                     = 0;                                                        // Reset Average
            AVGSum                  = 0;
            ChartIndex              = 0;
            HMCAnswer               = "";                                                       // Clear Receiver Buffer
            this.Invoke(new EventHandler(Stop_Button_Click));                                   // Stop current measurement
            this.Invoke(new EventHandler(Clear_Click));                                         // Clear Chart Content
            WriteHMCCommand(MeasureMode);
         }
        
         // DC Voltage Measurement Mode
         private void DCV_Button_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText(NotCon);
            else
            {
                MeasureMode = SetMeasDCV;                                                       // Set measuring mode
                ChangeMeasurementMode();                                                        // Change Measurement Mode
                Status_Text_Box.AppendText("Switched to DC Voltage measurement" + "\n");
                WireMode.Visible = false;                                                       // Disable PT Temperature sensor visibility
                PTSensor.Visible = false;                                                       // Disable Wire-Mode Option visibility
                MUnit = "V";                                                                    // Set measure value unit
                chart1.Series[0].Name = "DC Voltage";                                           // Changes Chart Series name
            }
        }

        // AC Voltage Measurement Mode
        private void ACV_Button_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText(NotCon);
            else
            {
                MeasureMode = SetMeasACV;           // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode
                Status_Text_Box.AppendText("Switched to AC Voltage measurement" + "\n");
                WireMode.Visible = false;           // Disable PT Temperature sensor visibility
                PTSensor.Visible = false;           // Disable Wire-Mode Option visibility
                MUnit = "V";
                chart1.Series[0].Name = "AC Voltage";
            }
        }

        // DC Current Measurment Mode
        private void DCI_Button_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText(NotCon);
            else
            {
                MeasureMode = SetMeasDCI;           // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode
                Status_Text_Box.AppendText("Switched to DC Current measurement" + "\n");
                WireMode.Visible = false;           // Disable PT Temperature sensor visibility
                PTSensor.Visible = false;           // Disable Wire-Mode Option visibility
                MUnit = "A";
                chart1.Series[0].Name = "DC Current";
            }
        }

        // AC Current Measurment Mode
        private void ACI_Button_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText(NotCon);
            else
            {
                MeasureMode = SetMeasACI;           // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode
                Status_Text_Box.AppendText("Switched to AC Current measurement" + "\n");
                WireMode.Visible = false;           // Disable PT Temperature sensor visibility
                PTSensor.Visible = false;           // Disable Wire-Mode Option visibility
                MUnit = "A";
                chart1.Series[0].Name = "AC Current";
            }
        }

        // Voltage Frequency Measurment Mode
        private void FreqV_Button_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText(NotCon);
            else
            {
                MeasureMode = SetMeasFreqV;         // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode
                Status_Text_Box.AppendText("Switched to Voltage Frequency measurement" + "\n");
                WireMode.Visible = false;           // Disable PT Temperature sensor visibility
                PTSensor.Visible = false;           // Disable Wire-Mode Option visibility
                MUnit = "Hz";
                chart1.Series[0].Name = "Voltage Frequency [Hz]";
            }
        }

        // Current Frequency Measurment Mode
        private void FreqI_Button_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText(NotCon);
            else
            {
                MeasureMode = SetMeasFreqC;         // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode
                Status_Text_Box.AppendText("Switched to Current Frequency measurement" + "\n");
                WireMode.Visible = false;           // Disable PT Temperature sensor visibility
                PTSensor.Visible = false;           // Disable Wire-Mode Option visibility
                MUnit = "Hz";
                chart1.Series[0].Name = "Current Frequency [Hz]";
            }
        }
            
        // Diode Measurment Mode
        private void DiodeButton_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText(NotCon);
            else
            {
                MeasureMode = SetMeasDiode;         // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode
                Status_Text_Box.AppendText("Switched to Diode measurement" + "\n");
                WireMode.Visible = false;           // Disable PT Temperature sensor visibility
                PTSensor.Visible = false;           // Disable Wire-Mode Option visibility
                chart1.Series[0].Name = "Diode Voltage [V]";
                MUnit = "V";
            }
        }

        // Resistance Measurment Mode
        private void R_Button_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText(NotCon);
            else
            {
                MeasureMode = SetMeas2WRes;         // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode
                Status_Text_Box.AppendText("Switched to 2-Wire Resistance measurement" + "\n");
                PTSensor.Visible = false;           // Disable PT Temperature sensor visibility
                WireMode.Visible = true;            // Enables Wire-Mode Option visibility
                chart1.Series[0].Name = "Resistance [Ω]";
                MUnit = "Ω";
            }
        }

        // 2-Wire Resistance Measurement Mode (Default)
        private void TwoWire_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText(NotCon);
            else
            {
                MeasureMode = SetMeas2WRes;         // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode"
                Status_Text_Box.AppendText("Switched to 2-Wire Resistance measurement" + "\n");
                chart1.Series[0].Name = "Resistance [Ω]";
                MUnit = "Ω";
            }
        }
        
        // 4-Wire Resistance Measurement Mode
        private void FourWire_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText(NotCon);
            else
            {
                MeasureMode = SetMeas4WRes;         // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode"
                Status_Text_Box.AppendText("Switched to 4-Wire Resistance measurement" + "\n");
                chart1.Series[0].Name = "Resistance [Ω]";
                MUnit = "Ω";
            }
        }

        // Capacitance Measurement Mode
        private void Cap_Button_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText(NotCon);
            else
            {
                MeasureMode = SetMeasCap;           // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode"
                Status_Text_Box.AppendText("Switched to Capacitance measurement" + "\n");
                PTSensor.Visible = false;           // Disable PT Temperature sensor visibility
                WireMode.Visible = false;           // Disable Wire-Mode Option visibility
                chart1.Series[0].Name = "Capacitance [µF]";
            }
        }
                                
        // Temperature Measurement Mode
        private void Temp_Button_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) Status_Text_Box.AppendText(NotCon);
            else
            {
                MeasureMode = SetMeasTemp;          // Set measuring mode
                ChangeMeasurementMode();            // Change Measurement Mode"
                Status_Text_Box.AppendText("Switched to Temperatue measurement" + "\n");
                PTSensor.Visible = true;            // Enables PT Temperature sensor visibility
                WireMode.Visible = true;            // Enables Wire-Mode Option visibility
                chart1.Series[0].Name = "Temperature [°C]";
                MUnit = " [°C]";
            }
        }

        // ------------------------ Select PT100, PT500, PT1000 Temperature Sensor -------------------------------------
        private void PT100_Click(object sender, EventArgs e)
        {
            Rnull = 100;   // 100 Ohm @ 0°C  
        }

        private void PT500_Click(object sender, EventArgs e)
        {
            Rnull = 500;   // 500 Ohm @ 0°C 
        }
       
        private void PT1000_Click(object sender, EventArgs e)
        {
            Rnull = 1000;  // 1000 Ohm @ 0°C
        }

        // ---------------------------------- ADC Conversion speed setting ---------------------------------------------
        private void Radio_Button_ADCSlow_CheckedChanged(object sender, EventArgs e)
        {
            WriteHMCCommand(SetADCSlow);
            if (WritingSucceed)
            {
                numericUpDown1.Value = 200;
                numericUpDown1.Minimum = 200;
            }
        }
        private void Radio_Button_ADCMed_CheckedChanged(object sender, EventArgs e)
        {
            WriteHMCCommand(SetADCMed);
            if (WritingSucceed)
            {
                numericUpDown1.Value = 150;
                numericUpDown1.Minimum = 100;
            }
        }
        private void Radio_Button_ADCFast_CheckedChanged(object sender, EventArgs e)
        {
            WriteHMCCommand(SetADCFast);
            if (WritingSucceed)
            {
                numericUpDown1.Value = 100;     // HMC8012 update rate in fast is up to 5ms but program is instable below 100 ms 
                numericUpDown1.Minimum = 100;
            }
        }
        // ------------------------------------- END MEASUREMENT SETTING ------------------------------------------------

        // ----------------------------------------- GRAPH CONTROLS -----------------------------------------------------            

        private void trackBar1_Scroll(object sender, EventArgs e)
        {
            if (ChartIndex > trackBar1.Maximum) trackBar1.Maximum = 300;
            else trackBar1.Maximum = ChartIndex;

            ZoomFactor = trackBar1.Value;
            if (Checkbox_Roll.Checked)
            {
                RollChart = true;
                chart1.ChartAreas[0].AxisX.ScaleView.Size = trackBar1.Value;        // Nur für den Roll-Modus
            }
            else
            {
                RollChart = false;
                chart1.ChartAreas[0].AxisX.ScaleView.Size = InitialZoomValueX;       // NaN
            }
        }

        private void Checkbox_Roll_CheckedChanged(object sender, EventArgs e)
        {
            ZoomFactor = trackBar1.Value;
            chart1.ChartAreas[0].AxisY.ScaleView.Size = InitialZoomValueY;         // Auto
            if (Checkbox_Roll.Checked)
            {
                RollChart = true;
                if (ChartIndex > trackBar1.Maximum)
                {
                    chart1.ChartAreas[0].AxisX.ScaleView.Size = ZoomFactor;//trackBar1.Maximum;        // Nur für den Roll-Modus
                    chart1.ChartAreas[0].AxisX.ScaleView.Position = ChartIndex - ZoomFactor; // Nur für Roll-Modus
                }
                else
                {
                    chart1.ChartAreas[0].AxisX.ScaleView.Size = ZoomFactor;
                    chart1.ChartAreas[0].AxisX.ScaleView.Position = ZoomFactor - ChartIndex;
                }
            }
            else
            {
                RollChart = false;
                chart1.ChartAreas[0].AxisX.ScaleView.Size = InitialZoomValueX;         // NaN
            }
        }

        private void chart1_MouseClick(object sender, MouseEventArgs e)
        {
            switch (e.Button)
            {
                case MouseButtons.Left:
                    // Left click
                    break;

                case MouseButtons.Right:
                    // Right click
                    chart1.ChartAreas[0].AxisY.ScaleView.Size = InitialZoomValueY;         // Auto
                    chart1.ChartAreas[0].AxisX.ScaleView.Size = InitialZoomValueX;         // NaN
                    break;
            }
        }

        private void chart1_MouseMove(object sender, MouseEventArgs e)
        {
            var pos = e.Location;
            if (prevPosition.HasValue && pos == prevPosition.Value)
                return;
            tooltip.RemoveAll();
            prevPosition = pos;
            var results = chart1.HitTest(pos.X, pos.Y, false, ChartElementType.DataPoint);
            foreach (var result in results)
            {
                if (result.ChartElementType == ChartElementType.DataPoint)
                {
                    var prop = result.Object as DataPoint;
                    if (prop != null)
                    {
                        var pointXPixel = result.ChartArea.AxisX.ValueToPixelPosition(prop.XValue);
                        var pointYPixel = result.ChartArea.AxisY.ValueToPixelPosition(prop.YValues[0]);

                        // check if the cursor is really close to the point (5 pixels around the point)
                        if (Math.Abs(pos.X - pointXPixel) < 10 &&
                            Math.Abs(pos.Y - pointYPixel) < 10)
                        {
                            tooltip.Show("X=" + prop.XValue + ", Y=" + prop.YValues[0].ToString("0.#####0") + " " + MUnit, this.chart1,
                                            pos.X, pos.Y - 15);
                        }
                    }
                }
            }
        }

        private void numericUpDown1_ValueChanged(object sender, EventArgs e)
        {
            DataReadTimer.Interval = Convert.ToInt32(numericUpDown1.Value);     // Sets the timer interval according to numeric input field
        }
    }
}
